"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_transformer_core_1 = require("graphql-transformer-core");
const graphql_mapping_template_1 = require("graphql-mapping-template");
const graphql_transformer_common_1 = require("graphql-transformer-common");
class DefaultValueTransformer extends graphql_transformer_core_1.Transformer {
    constructor() {
        super('DefaultValueTransformer', `directive @default(value: String) on FIELD_DEFINITION`);
        /**
         * When a field is annotated with @default(value: "value") the mapping template will default if not set by the user.
         *
         * Usage:
         *
         * type Post @model {
         *   id: ID!
         *   title: String @default(value: "Hello World")
         *   version: Int!
         * }
         *
         * If a value is set by the client, use that, otherwise
         * default to what is in the value of the @default directive.
         */
        this.field = (parent, def, directive, ctx) => {
            // @versioned may only be used on types that are also @model
            const modelDirective = parent.directives.find((dir) => dir.name.value === 'model');
            if (!modelDirective) {
                throw new graphql_transformer_core_1.InvalidDirectiveError('Fields annotated with @default must also belong to types annotated with @model.');
            }
            if (directive.arguments.length != 1) {
                throw new graphql_transformer_core_1.InvalidDirectiveError('Fields annotated with @default must declare a single argument and value.');
            }
            let defaultArgValue;
            let fieldType;
            if (directive.arguments[0].value.value) {
                defaultArgValue = directive.arguments[0].value.value;
            }
            else {
                throw new graphql_transformer_core_1.InvalidDirectiveError('Directive for @default must declare a value property');
            }
            if (def.type.name.value) {
                fieldType = def.type.name.value;
            }
            else {
                throw new graphql_transformer_core_1.InvalidDirectiveError('Directive for @default must declare a value property');
            }
            let defaultValue;
            switch (fieldType) {
                case "String":
                    defaultValue = `"${defaultArgValue}"`;
                    break;
                case "Int":
                    defaultValue = `${defaultArgValue}`;
                    break;
                default:
                    throw new graphql_transformer_core_1.InvalidDirectiveError('Fields annotated with @default must declare a single argument and value.');
            }
            const typeName = parent.name.value;
            const fieldName = def.name.value;
            this.augmentCreateMutation(ctx, typeName, fieldName, defaultValue);
        };
    }
    /**
     * Set the field to the default value on create if not set otherwise.
     * @param ctx
     * @param typeName
     * @param fieldName
     * @param defaultValue
     */
    augmentCreateMutation(ctx, typeName, fieldName, defaultValue) {
        const snippet = graphql_mapping_template_1.printBlock(`Setting "${fieldName}" to default to ${defaultValue}`)(graphql_mapping_template_1.iff(graphql_mapping_template_1.raw(`$util.isNull($ctx.args.input.${fieldName})`), graphql_mapping_template_1.qref(`$ctx.args.input.put("${fieldName}", ${defaultValue})`)));
        const mutationResolverLogicalId = graphql_transformer_common_1.ResolverResourceIDs.DynamoDBCreateResolverResourceID(typeName);
        const resolver = ctx.getResource(mutationResolverLogicalId);
        if (resolver) {
            resolver.Properties.RequestMappingTemplate = snippet + '\n\n' + resolver.Properties.RequestMappingTemplate;
            ctx.setResource(mutationResolverLogicalId, resolver);
        }
    }
}
exports.DefaultValueTransformer = DefaultValueTransformer;
//# sourceMappingURL=DefaultValueTransformer.old.js.map